@@ -144,7 +144,7 @@ mix_ins_to_word(const mix_ins_t *ins);
 extern mix_ins_id_t
 mix_word_to_ins(mix_word_t w, mix_ins_t *ins);
 
-#define mix_word_set_address(word,addr) (word) |= ((addr)<<18) 
+#define mix_word_add_address(word,addr) (word) |= ((addr)<<18) 
 
 /* decompose an instruction codified in a word into its parts */
 #define mix_get_ins_address(word) ((mix_address_t)((word)>>18))
