@@ -186,19 +186,19 @@ undef_warning_ (gpointer symbol, gpointer value, gpointer data)
   mix_parser_log_error ((mix_parser_t *)data, MIX_PERR_UNDEF_SYM, 1,
 			(const gchar *)symbol, TRUE);
   /* move the symbol to the symbol table */
-  mix_symbol_table_insert_static (((mix_parser_t *)data)->symbol_table, symbol, 
-				  MIX_WORD_ZERO);
+  mix_symbol_table_insert_static (((mix_parser_t *)data)->symbol_table,
+				  symbol, MIX_WORD_ZERO);
   return TRUE;
 }
 
 static void
-update_future_refs_ (mix_parser_t *parser, const gchar *name)
+update_future_refs_value_ (mix_parser_t *parser, const gchar *name,
+			   mix_short_t value)
 {
   GSList *list = NULL;
   gpointer key;
   
   g_assert (parser != NULL && name != NULL);
-  
   if ( g_hash_table_lookup_extended (parser->future_refs, name, &key, 
 				     (gpointer *)&list) )
     {
@@ -209,7 +209,13 @@ update_future_refs_ (mix_parser_t *parser, const gchar *name)
 	  node = 
 	    (ins_node_ *)g_tree_lookup (parser->ins_table,tmp->data);
 	  g_assert (node);
-	  mix_word_set_address (node->ins, parser->loc_count);
+	  if (mix_get_ins_address (node->ins) == 1) {
+	    value = mix_short_negative (value);
+	    node->ins = mix_word_set_field (node->ins,
+					    MIX_WORD_ZERO,
+					    mix_fspec_new (1,2));
+	  }
+	  mix_word_add_address (node->ins, value);
 	  g_tree_insert (parser->ins_table, tmp->data, (gpointer)node);
 	  tmp = g_slist_next (tmp);
 	}
@@ -219,6 +225,9 @@ update_future_refs_ (mix_parser_t *parser, const gchar *name)
     }
 }
 
+#define update_future_refs_(parser,name) \
+  update_future_refs_value_(parser, name, (parser)->loc_count);
+
 static void
 update_ls_ (gpointer symbol, gpointer value, gpointer parser)
 { /* add an instruction on current location and update refs to it */
@@ -350,7 +359,7 @@ mix_parser_write_listing (mix_parser_t *parser, const gchar *list_file)
   fprintf (file, _("\n*** Start address: %d\n"), 
 	   mix_short_magnitude (parser->start));
   fprintf (file, _("\n*** Symbol table\n"));
-  mix_symbol_table_print (parser->symbol_table, MIX_SYM_ROWS, file);
+  mix_symbol_table_print (parser->symbol_table, MIX_SYM_ROWS, file, TRUE);
   fprintf (file, _("\n*** End of summary ***\n"));
   mix_file_delete (mfile);
   return parser->status;
@@ -392,14 +401,20 @@ mix_parser_err_t
 mix_parser_define_symbol_here (mix_parser_t *parser, const gchar *name)
 {
   mix_word_t value = mix_short_to_word_fast (parser->loc_count);
-  
+  return mix_parser_define_symbol_value (parser, name, value);
+}
+
+mix_parser_err_t
+mix_parser_define_symbol_value (mix_parser_t *parser, const gchar *name,
+				mix_word_t value)
+{
   g_assert (parser != NULL && name != NULL);  
   
   switch (mix_symbol_table_add (parser->symbol_table, name, value))
     {
     case MIX_SYM_OK: 
       if ( parser->status == MIX_PERR_NOCOMP ) 
-	update_future_refs_ (parser, name);
+	update_future_refs_value_ (parser, name, value);
       return MIX_PERR_OK;
     case MIX_SYM_LONG: return MIX_PERR_LONG_SYMBOL;
     case MIX_SYM_DUP: return MIX_PERR_DUP_SYMBOL;
@@ -427,7 +442,8 @@ mix_parser_set_future_ref (mix_parser_t *parser, const gchar *name)
 
 /* Redefine the value of a local symbol as the current loc_count */
 void
-mix_parser_manage_local_symbol (mix_parser_t *parser, const gchar *name)
+mix_parser_manage_local_symbol (mix_parser_t *parser, const gchar *name,
+				mix_short_t value)
 {
   gchar ref[3];
   ref[2] = 0;
@@ -444,10 +460,10 @@ mix_parser_manage_local_symbol (mix_parser_t *parser, const gchar *name)
       ref[0] = name[0];
       ref[1] = 'F';
       if ( parser->status == MIX_PERR_NOCOMP ) 
-	update_future_refs_ (parser, ref);
+	update_future_refs_value_ (parser, ref, value);
       ref[1] = 'B';
       mix_symbol_table_insert (parser->symbol_table, ref, 
-			       mix_short_to_word_fast (parser->loc_count));
+			       mix_short_to_word_fast (value));
       break;
     default:
       return;
