@@ -19,7 +19,7 @@
  *  
  */
 
-
+#include <ctype.h> /* isdigit */
 #include "mix_symbol_table.h"
 
 /* Create an empty table */
@@ -111,26 +111,41 @@ mix_symbol_table_is_defined(const mix_symbol_table_t *table, const gchar *sym)
 }
 
 /* Print table */
+#define is_local_sym_(sym) \
+  ((sym) && (strlen(sym)==2) && (sym[1] == 'B') && isdigit(sym[0]))
+
+static gboolean skip_ = FALSE;
+
 static void
 print_sym_rows_(gpointer symbol, gpointer value, gpointer file)
 {
-  mix_word_t word = (mix_word_t)GPOINTER_TO_UINT(value);
-  fprintf((FILE *)file, "%s:  %s%ld\n", (gchar *)symbol, 
-	  mix_word_is_negative(word)? "-":"", mix_word_magnitude(word));
+  char *s = (char *)symbol;
+  if (skip_ && !is_local_sym_(s)) {
+    mix_word_t word = (mix_word_t)GPOINTER_TO_UINT(value);
+    fprintf((FILE *)file, "%s:  %s%ld\n", s, 
+	    mix_word_is_negative(word)? "-":"",
+	    mix_word_magnitude(word));
+  }
 }
 
 static void
 print_sym_line_(gpointer symbol, gpointer value, gpointer file)
 {
-  mix_word_t word = (mix_word_t)GPOINTER_TO_UINT(value);
-  fprintf((FILE *)file, ",%s =%s%ld", (gchar *)symbol, 
-	  mix_word_is_negative(word)? "-":"", mix_word_magnitude(word));
+  char *s = (char *)symbol;
+  if (skip_ && !is_local_sym_(s)) {
+    mix_word_t word = (mix_word_t)GPOINTER_TO_UINT(value);
+    fprintf((FILE *)file, ",%s =%s%ld", s, 
+	    mix_word_is_negative(word)? "-":"",
+	    mix_word_magnitude(word));
+  }
 }
 
 void
-mix_symbol_table_print(const mix_symbol_table_t *table, gint mode, FILE *file)
+mix_symbol_table_print(const mix_symbol_table_t *table, gint mode,
+		       FILE *file, gboolean skiplocal)
 {
   GHFunc func = (mode == MIX_SYM_LINE)? print_sym_line_ : print_sym_rows_;
+  skip_ = skiplocal;
   if ( table != NULL )
     g_hash_table_foreach((GHashTable *)table, func, (gpointer)file);
   if ( mode == MIX_SYM_LINE ) putc(';', file); /* to mark end-of-table */
