@@ -347,7 +347,7 @@ cmd_psym_ (char *arg)
 	printf (_("%s: symbol not defined\n"), arg);
     }
   else
-    mix_symbol_table_print (table, MIX_SYM_ROWS, stdout);
+    mix_symbol_table_print (table, MIX_SYM_ROWS, stdout, FALSE);
       
   return TRUE;
 }
